//
//  RTcmixPlayer.h
//  subclass this singleton for iRTcmix audio work
//	will function in both AudioQueueBuffer and AudioUnit mode
//
//  Created by Brad Garton on 7/17/09.
//	additional work by Damon Holzborn
//

#include <AudioToolbox/AudioToolbox.h>
#import <UIKit/UIKit.h>
#import "RTcmixScore.h"

// RTcmix external functions called from the RTcmixPlayer
extern int maxmsp_rtsetparams(float sr, int nchans, int vecsize, float *mm_inbuf, float *mm_outbuf, char *mm_errbuf);
extern void pullTraverse(short *inbuf, short *outbuf); // we pass in pointers to the audio bufs
extern int check_bang();
extern void pfield_set(int inlet, float pval);
extern void buffer_set(char *bufname, float *bufstart, int nframes, int nchans, int modtime);
char errbuf[1024]; // not used right now, but necessary for RTcmix error-writing
extern int rtcmixmain();
extern int parse_score(char *thebuf, int buflen);
extern void flush_sched();
extern int check_vals(float *thevals);
int maxmessage_nvals;
float maxmessage_vals[1024];

// audio setup
int numberAudioDataBuffers;
int vectorSize;
int numberOfChannels;

@interface RTcmixPlayer : NSObject {
	// Audio Queue
	AudioQueueBufferRef				buffers[32];	// the audio queue buffers for the audio queue
	UInt32							bufferByteSize;						// the number of bytes to use in each audio queue buffer
	BOOL							audioPlayerShouldStopImmediately;	// this is used for 'async' vs. 'sync' shutdown of audio
	AudioQueueRef					queueObject;						// the audio queue object being used for playback
	AudioStreamBasicDescription		audioFormat;						// used in setPlaybackStuff to set up audio characteristics
	
	// AudioUnit
	AURenderCallbackStruct			inputProc;
	AudioUnit						rioUnit;							// the RemoteIO Audio Unit
	short							*rtcmixOutputBuffer;
	int								outputBufferIndex;
	short							*rtcmixInputBuffer;
	int								inputBufferIndex;
	int								audioUnitFlag;						// 1 if running RemoteIO, 0 otherwise
	int								audioInputFlag;						// 1 if RemoteIO input enabled
}

// Audio Queue
@property (readwrite)			UInt32						bufferByteSize;
@property (readwrite)			BOOL						audioPlayerShouldStopImmediately;
@property (readwrite)			AudioQueueRef				queueObject;
@property (readwrite)			AudioStreamBasicDescription	audioFormat;

// AudioUnit
@property (nonatomic, assign)	AURenderCallbackStruct		inputProc;
@property (nonatomic, assign)	AudioUnit					rioUnit;
@property (readwrite)			short						*rtcmixOutputBuffer;
@property (readwrite)			int							outputBufferIndex;
@property (readwrite)			short						*rtcmixInputBuffer;
@property (readwrite)			int							inputBufferIndex;
@property (readwrite)			int							audioUnitFlag;
@property (readwrite)			int							audioInputFlag;

// Data and Communication
- (void)parseScoreWithRTcmixScore:(RTcmixScore *)score;
- (void)parseScoreWithNSString:(NSString *)score;
- (void)setInlet:(int)inlet withValue:(Float32)value;
- (void)maxBang;
- (void)maxMessage;
- (void)flushAllScripts;
- (int)setSampleBuffer:(NSString *)bufferName withSoundFile:(NSString *)soundFileName;

// Setup
- (void)startAudioQueuePlayer;
- (void)startAUPlayerWithInputFlag:(int)inputFlag;	// init RTcmixPlayer using RemoteIO AudioUnit
- (void)setupAudioSettings;
- (void)setupPlayback;
- (int)setupRemoteIO;							// for AudioUnit
- (void)setupPlaybackAudioQueueObject;
- (void)setupAudioQueueBuffers;

// Audio Utility
- (void)playAudio;
- (void)stopAudio;
- (void)pauseAudio;
- (void)resumeAudio;
- (void)disposeAudioQueue;
- (void)restartAudioQueue;

// Singleton
+ (id)sharedManager;

// Playback and Interrupt Callbacks
static void playbackCallback (void *inUserData, AudioQueueRef inAudioQueue, AudioQueueBufferRef bufferReference);
static OSStatus	rtcmixPerformCallback (void *inRefCon, AudioUnitRenderActionFlags 	*ioActionFlags, 
									   const AudioTimeStamp *inTimeStamp, UInt32 inBusNumber, UInt32 inNumberFrames, 
									   AudioBufferList *ioData);
void interruptionListenerCallback (void *inUserData, UInt32 interruptionState );
void propListenerCallback(void *inClientData, AudioSessionPropertyID inID, UInt32 inDataSize,const void *inData);

@end
